#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>

#include "coords.h"
#include "gfx.h"
#include "geom.h"
#include "settings.h"

View::View(CartCoord icentre, float izoom, float iangle) :
    centre(icentre), zoom(izoom), angle(iangle) {}

ScreenCoord View::coord(const CartCoord &c) const
{
    RelCartCoord d( (c-centre).rotated(angle) );
    return ScreenCoord(
	    screenGeom.centre.x+(int)(d.dx*zoom),
	    screenGeom.centre.y-(int)(d.dy*zoom));
}

bool View::inView(const CartCoord &c, float in) const
{
    RelCartCoord d = c-centre;
    return ( (zoom*zoom)*(d.dx*d.dx+d.dy*d.dy) <= (screenGeom.rad-in)*(screenGeom.rad-in) );
}

int Line::draw(SDL_Surface* surface, const View& view, View* boundView, bool noAA)
{
    ScreenCoord s = view.coord(start);
    ScreenCoord e = view.coord(end);
    if (!boundView || (boundView->inView(start) && boundView->inView(end)))
	return ( (!noAA && settings.useAA) ? aalineColor : lineColor )
	    (surface, s.x, s.y, e.x, e.y, colour);
    else
	return 0;
}

int Circle::draw(SDL_Surface* surface, const View& view, View* boundView, bool noAA)
{
    ScreenCoord c = view.coord(centre);
    if (!boundView || boundView->inView(centre, r*view.zoom))
	return (filled ? filledCircleColor : 
		(!noAA && settings.useAA) ? aacircleColor : circleColor)
	    (surface, c.x, c.y, (int)(r*view.zoom), colour);
    else
	return 0;
}

int Polygon::draw(SDL_Surface* surface, const View& view, View* boundView, bool noAA)
{
    Sint16 sx[n], sy[n];

    for (int i=0; i<n; i++)
    {
	if (boundView && !boundView->inView(points[i]))
	    return 0;
    }

    for (int i=0; i<n; i++)
    {
	ScreenCoord s = view.coord(points[i]);
	sx[i] = s.x;
	sy[i] = s.y;
    }
    return ( filled ? filledPolygonColor : 
	    (!noAA && settings.useAA) ? aapolygonColor : polygonColor )(
		surface, sx, sy, n, colour);
}


void draw_cross(SDL_Surface* surface, ScreenCoord s)
{
    lineColor(surface, s.x-5,s.y-5,s.x+5,s.y+5, 0xffffffff);
    lineColor(surface, s.x-5,s.y+5,s.x+5,s.y-5, 0xffffffff);
}
