#include <algorithm>
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>

#include "gfx.h"
#include "shot.h"

void Shot::update(int time)
{
    pos += vel*time;
}

void Shot::draw(SDL_Surface* surface, const View& view, View* boundView)
{
    Uint32 basecolour;

    switch (weight)
    {
	case 3: basecolour = 0xff000000;
		break;
	case 2: basecolour = 0xffff0000;
		break;
	default: basecolour = 0x00ff0000;
    }

    Line(pos, pos + vel*-50,
	    basecolour + (super ? 0xb0 : 0x70)
	).draw(surface, view, boundView);

    if (!boundView || boundView->inView(pos))
    {
	ScreenCoord e = view.coord(pos);
	pixelColor(surface, e.x, e.y, basecolour + (super ? 0xff : 0xe0));
    }
}

int Shot::hit(int damage)
{
    int lost = std::min(weight, damage);
    weight -= lost;
    if (weight == 0)
	dead = 1;
    return lost;
}

int Shot::die()
{
    return hit(weight);
}
